.setcpu "65816"
.smart
.feature force_range

.include "Macros.asm"

; See 2input.asm
.import JoyInit, GetInput

; Used by 2input.asm
.exportzp Joy1Data, Joy2Data, Joy1Press, Joy2Press

; Used by Header.asm
.export VBlank, Main

;
; Game variables, located within 6502 zero page ($00-ff)
; 65816 has direct page which can range from $0000-ffff in region, but ca65
; only seems to understand "zp" being up to 256 bytes in size.
;
; See lorom256k.cfg and lda/tcd code in _reset
;
.segment "ZEROPAGE"

Joy1Data:                  .res 2
Joy2Data:                  .res 2
Joy1Press:                 .res 2
Joy2Press:                 .res 2

;
; SNES RAM area, $0200-1fff
; 
; See lorom256k.cfg
;

.segment "BSS"

.segment "BSS7E"

;
; Program/code area
;
.segment "CODE"

.proc Main
  ;
  ; Initialize SNES, which includes setting up CPU registers properly,
  ; zeroing out RAM contents, setting up SNES registers per official
  ; Nintendo documentation, and many other things.
  ;
  InitializeSNES

  jsr JoyInit      ; setup joypads

  jsr SetupVideo   ; setup the video modes and such, then turn on the screen

  ; Enable NMI (when interrupts get re-enabled) and joypad reading
  lda #$81
  sta $4200

  cli              ; Restore interrupts -- we're all ready to go!

InfiniteLoop:
  wai                         ; Wait for interrupts to finish (NMI/VBlank)
  jmp InfiniteLoop            ; Do this forever

.endproc

;=========================================================================================
; VBlank
;=========================================================================================

.proc VBlank
  rep #$30   ; A=16, X/Y=16
  pha
  phx
  phy
  phd
  phb
  php

  jsr GetInput    ; update joypad data

  lda $4210       ; clear NMI Flag

  plp
  plb
  pld
  ply
  plx
  pla
  rti
.endproc

;=========================================================================================
; SetupVideo
;=========================================================================================

.proc SetupVideo
  php
  rep #$30   ; A=16, X/Y=16
  sep #$20   ; A=8

  stz $211b
  lda #$01
  sta $211b
  stz $211c
  stz $211c
  stz $211d
  stz $211d
  stz $211e
  sta $211e
  lda #$80
  sta $211f
  stz $211f
  sta $2120
  stz $2120

  plp
  rts
.endproc

;=========================================================================================
; Segment for our binary data
;=========================================================================================
.segment "RODATA"
;=========================================================================================